#!/usr/bin/env python3
"""
Final improved Hatena Bookmark scraper with accurate URL extraction
"""

import os
import json
import re
import time
import requests
from bs4 import BeautifulSoup
from typing import List, Dict, Optional
import logging
from urllib.parse import unquote, parse_qs, urlparse

# Setup logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)


class FinalHatenaBookmarkScraper:
    def __init__(self):
        self.base_url = "https://bookmark.hatenastaff.com"
        self.archive_url = f"{self.base_url}/archive/category/ランキング"
        self.headers = {
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36"
        }
        self.session = requests.Session()
        self.session.headers.update(self.headers)

        # Create data directories
        self.data_dir = os.path.join(os.path.dirname(__file__), "..", "data")
        self.week_dir = os.path.join(self.data_dir, "week")
        self.month_dir = os.path.join(self.data_dir, "month")

        os.makedirs(self.week_dir, exist_ok=True)
        os.makedirs(self.month_dir, exist_ok=True)

    def get_page(self, url: str) -> Optional[BeautifulSoup]:
        """Get a page and return BeautifulSoup object"""
        try:
            logger.info(f"Fetching: {url}")
            response = self.session.get(url, timeout=30)
            response.raise_for_status()
            return BeautifulSoup(response.content, "html.parser")
        except Exception as e:
            logger.error(f"Error fetching {url}: {e}")
            return None

    def extract_original_url_from_hatena_link(self, hatena_bookmark_url: str) -> str:
        """Extract original URL from Hatena bookmark URL"""
        try:
            parsed = urlparse(hatena_bookmark_url)
            if "b.hatena.ne.jp" in parsed.netloc:
                query_params = parse_qs(parsed.query)
                if "url" in query_params:
                    return unquote(query_params["url"][0])
        except Exception:
            pass
        return ""

    def extract_ranking_data_from_table(
        self, url: str, expected_count: int
    ) -> List[Dict[str, str]]:
        """Extract ranking data from the table structure"""
        soup = self.get_page(url)
        if not soup:
            return []

        entries = []

        # Find the ranking table
        table = soup.find("table")
        if not table:
            logger.warning(f"No table found in {url}")
            return []

        rows = table.find_all("tr")
        logger.info(f"Found table with {len(rows)} rows")

        for row_index, row in enumerate(rows):
            if row_index == 0:  # Skip header row
                continue

            cells = row.find_all(["td", "th"])
            if len(cells) < 2:
                continue

            # Extract rank from first cell
            rank_cell = cells[0]
            rank_text = rank_cell.get_text().strip()
            rank_match = re.search(r"(\d+)位", rank_text)

            if not rank_match:
                continue

            rank = int(rank_match.group(1))

            # Extract title and URLs from second cell
            title_cell = cells[1]

            # Get the title text (clean version)
            title = title_cell.get_text().strip()

            # Find all links in the title cell
            links = title_cell.find_all("a", href=True)

            original_url = ""
            hatena_url = ""

            for link in links:
                href = link.get("href")
                if not href:
                    continue

                if "b.hatena.ne.jp/entry" in href:
                    # This is the hatena bookmark URL
                    hatena_url = href
                    # Extract original URL from the hatena bookmark URL
                    if not original_url:
                        original_url = self.extract_original_url_from_hatena_link(href)
                elif (
                    href.startswith("http")
                    and "hatenastaff.com" not in href
                    and "hatena.ne.jp" not in href
                ):
                    # This is likely the original URL
                    original_url = href

            if rank <= expected_count:
                entry = {
                    "rank": rank,
                    "title": title,
                    "url": original_url,
                    "hatena_url": hatena_url,
                }
                entries.append(entry)

                # Debug log for first few entries
                if len(entries) <= 5:
                    logger.debug(
                        f"Entry {rank}: {title[:50]}... | URL: {original_url[:60]}... | Hatena: {hatena_url[:60]}..."
                    )

        # Sort by rank and ensure we have the right count
        entries.sort(key=lambda x: x["rank"])
        entries = entries[:expected_count]

        logger.info(f"Successfully extracted {len(entries)} entries from {url}")
        return entries

    def extract_archive_links(self) -> Dict[str, List[tuple]]:
        """Extract weekly and monthly ranking article links from archive page"""
        soup = self.get_page(self.archive_url)
        if not soup:
            return {"weekly": [], "monthly": []}

        weekly_links = []
        monthly_links = []

        # Find all links in the archive page
        for link in soup.find_all("a", href=True):
            href = link.get("href")
            text = link.get_text().strip()

            if not href or not text:
                continue

            # Make absolute URL
            if href.startswith("/"):
                href = self.base_url + href

            # Check for weekly ranking pattern
            if "今週のはてなブックマーク数ランキング（2025年" in text:
                match = re.search(r"2025年(\d+)月第(\d+)週", text)
                if match:
                    month = int(match.group(1))
                    week = int(match.group(2))
                    if 1 <= month <= 5:
                        weekly_links.append((href, month, week))

            # Check for monthly ranking pattern
            elif "月間はてなブックマーク数ランキング（2025年" in text:
                match = re.search(r"2025年(\d+)月", text)
                if match:
                    month = int(match.group(1))
                    if 1 <= month <= 5:
                        monthly_links.append((href, month))

        logger.info(
            f"Found {len(weekly_links)} weekly links and {len(monthly_links)} monthly links"
        )
        return {"weekly": weekly_links, "monthly": monthly_links}

    def rescrape_all_data(self):
        """Re-scrape all data with improved URL extraction"""
        logger.info("Starting complete re-scraping with improved URL extraction...")

        # Get all archive links
        links = self.extract_archive_links()

        # Re-scrape all weekly data
        logger.info("Re-scraping all weekly ranking data...")
        for url, month, week in links["weekly"]:
            filename = f"2025-{month:02d}-{week}.json"
            filepath = os.path.join(self.week_dir, filename)

            logger.info(f"Re-scraping weekly data: {month}月第{week}週 -> {filename}")
            entries = self.extract_ranking_data_from_table(url, 30)

            if len(entries) == 30:
                with open(filepath, "w", encoding="utf-8") as f:
                    json.dump(entries, f, ensure_ascii=False, indent=2)
                logger.info(
                    f"✓ Successfully saved {len(entries)} entries to {filename}"
                )

                # Verify data quality
                urls_found = sum(1 for entry in entries if entry["url"])
                hatena_urls_found = sum(1 for entry in entries if entry["hatena_url"])
                logger.info(
                    f"  Data quality: {urls_found}/30 original URLs, {hatena_urls_found}/30 hatena URLs"
                )
            else:
                logger.error(
                    f"✗ Failed to get complete data for {filename}: got {len(entries)} entries instead of 30"
                )

            time.sleep(2)  # Be respectful to the server

        # Re-scrape all monthly data
        logger.info("Re-scraping all monthly ranking data...")
        for url, month in links["monthly"]:
            filename = f"2025-{month:02d}.json"
            filepath = os.path.join(self.month_dir, filename)

            logger.info(f"Re-scraping monthly data: {month}月 -> {filename}")
            entries = self.extract_ranking_data_from_table(url, 50)

            if len(entries) == 50:
                with open(filepath, "w", encoding="utf-8") as f:
                    json.dump(entries, f, ensure_ascii=False, indent=2)
                logger.info(
                    f"✓ Successfully saved {len(entries)} entries to {filename}"
                )

                # Verify data quality
                urls_found = sum(1 for entry in entries if entry["url"])
                hatena_urls_found = sum(1 for entry in entries if entry["hatena_url"])
                logger.info(
                    f"  Data quality: {urls_found}/50 original URLs, {hatena_urls_found}/50 hatena URLs"
                )
            else:
                logger.error(
                    f"✗ Failed to get complete data for {filename}: got {len(entries)} entries instead of 50"
                )

            time.sleep(2)  # Be respectful to the server

        logger.info("Complete re-scraping finished!")

    def verify_data_quality(self):
        """Verify the quality of all scraped data"""
        logger.info("Verifying data quality...")

        # Check weekly data
        import glob

        week_files = glob.glob(os.path.join(self.week_dir, "*.json"))
        month_files = glob.glob(os.path.join(self.month_dir, "*.json"))

        print("\n=== WEEKLY DATA QUALITY REPORT ===")
        total_weekly_entries = 0
        total_weekly_urls = 0
        total_weekly_hatena_urls = 0

        for file_path in sorted(week_files):
            filename = os.path.basename(file_path)
            try:
                with open(file_path, "r", encoding="utf-8") as f:
                    data = json.load(f)

                count = len(data)
                urls = sum(1 for entry in data if entry.get("url"))
                hatena_urls = sum(1 for entry in data if entry.get("hatena_url"))

                total_weekly_entries += count
                total_weekly_urls += urls
                total_weekly_hatena_urls += hatena_urls

                status = "✓" if count == 30 else "✗"
                print(
                    f"{status} {filename}: {count} entries, {urls} URLs, {hatena_urls} hatena URLs"
                )

            except Exception as e:
                print(f"✗ {filename}: Error - {e}")

        print(
            f"\nWeekly Summary: {total_weekly_entries} entries, {total_weekly_urls} URLs, {total_weekly_hatena_urls} hatena URLs"
        )

        print("\n=== MONTHLY DATA QUALITY REPORT ===")
        total_monthly_entries = 0
        total_monthly_urls = 0
        total_monthly_hatena_urls = 0

        for file_path in sorted(month_files):
            filename = os.path.basename(file_path)
            try:
                with open(file_path, "r", encoding="utf-8") as f:
                    data = json.load(f)

                count = len(data)
                urls = sum(1 for entry in data if entry.get("url"))
                hatena_urls = sum(1 for entry in data if entry.get("hatena_url"))

                total_monthly_entries += count
                total_monthly_urls += urls
                total_monthly_hatena_urls += hatena_urls

                status = "✓" if count == 50 else "✗"
                print(
                    f"{status} {filename}: {count} entries, {urls} URLs, {hatena_urls} hatena URLs"
                )

            except Exception as e:
                print(f"✗ {filename}: Error - {e}")

        print(
            f"\nMonthly Summary: {total_monthly_entries} entries, {total_monthly_urls} URLs, {total_monthly_hatena_urls} hatena URLs"
        )

        total_entries = total_weekly_entries + total_monthly_entries
        total_urls = total_weekly_urls + total_monthly_urls
        total_hatena_urls = total_weekly_hatena_urls + total_monthly_hatena_urls

        print("\n=== OVERALL SUMMARY ===")
        print(f"Total entries: {total_entries}")
        print(f"Total original URLs: {total_urls}")
        print(f"Total hatena URLs: {total_hatena_urls}")
        print(f"URL completion rate: {total_urls / total_entries * 100:.1f}%")
        print(
            f"Hatena URL completion rate: {total_hatena_urls / total_entries * 100:.1f}%"
        )


if __name__ == "__main__":
    scraper = FinalHatenaBookmarkScraper()

    # Re-scrape all data with improved extraction
    scraper.rescrape_all_data()

    # Verify data quality
    scraper.verify_data_quality()



#!/usr/bin/env python3
"""
Improved Hatena Bookmark Weekly/Monthly Ranking Data Scraper

This version focuses on better URL extraction by analyzing the actual HTML structure
of the ranking pages.
"""

import os
import json
import re
import time
import requests
from bs4 import BeautifulSoup
from typing import List, Dict, Optional
import logging
from urllib.parse import urljoin

# Setup logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)


class ImprovedHatenaBookmarkScraper:
    def __init__(self):
        self.base_url = "https://bookmark.hatenastaff.com"
        self.archive_url = f"{self.base_url}/archive/category/ランキング"
        self.headers = {
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36"
        }
        self.session = requests.Session()
        self.session.headers.update(self.headers)

        # Create data directories
        self.data_dir = os.path.join(os.path.dirname(__file__), "..", "data")
        self.week_dir = os.path.join(self.data_dir, "week")
        self.month_dir = os.path.join(self.data_dir, "month")

        os.makedirs(self.week_dir, exist_ok=True)
        os.makedirs(self.month_dir, exist_ok=True)

    def get_page(self, url: str) -> Optional[BeautifulSoup]:
        """Get a page and return BeautifulSoup object"""
        try:
            logger.info(f"Fetching: {url}")
            response = self.session.get(url, timeout=30)
            response.raise_for_status()
            return BeautifulSoup(response.content, "html.parser")
        except Exception as e:
            logger.error(f"Error fetching {url}: {e}")
            return None

    def extract_ranking_data_improved(
        self, url: str, expected_count: int
    ) -> List[Dict[str, str]]:
        """Extract ranking data with improved URL detection"""
        soup = self.get_page(url)
        if not soup:
            return []

        entries = []

        # Get the main content area
        content = soup.find("div", class_="entry-content") or soup.find(
            "div", class_="hatena-body"
        )
        if not content:
            content = soup

        # Look for table or list structures that contain ranking data
        tables = content.find_all("table")
        if tables:
            # Process table-based ranking
            for table in tables:
                rows = table.find_all("tr")
                for row in rows:
                    cells = row.find_all(["td", "th"])
                    if len(cells) >= 2:
                        # Try to extract rank and title
                        rank_text = cells[0].get_text().strip()
                        title_cell = cells[1] if len(cells) > 1 else cells[0]

                        # Extract rank number
                        rank_match = re.search(r"(\d+)", rank_text)
                        if rank_match:
                            rank = int(rank_match.group(1))

                            # Extract title and URL
                            title = title_cell.get_text().strip()

                            # Look for links in the title cell
                            link = title_cell.find("a", href=True)
                            url_link = ""
                            hatena_url = ""

                            if link:
                                href = link.get("href")
                                if href:
                                    if (
                                        "bookmark.hatena.ne.jp" in href
                                        or "b.hatena.ne.jp" in href
                                    ):
                                        hatena_url = href
                                    elif href.startswith("http"):
                                        url_link = href
                                    else:
                                        # Try to construct full URL
                                        full_url = urljoin(url, href)
                                        if "bookmark.hatena" in full_url:
                                            hatena_url = full_url
                                        else:
                                            url_link = full_url

                            if title and rank <= expected_count:
                                entries.append(
                                    {
                                        "rank": rank,
                                        "title": title,
                                        "url": url_link,
                                        "hatena_url": hatena_url,
                                    }
                                )

        # If no table found, try alternative extraction methods
        if not entries:
            # Method 2: Look for numbered list patterns
            text_content = content.get_text()
            lines = [line.strip() for line in text_content.split("\n") if line.strip()]

            # Find all links in the content
            all_links = content.find_all("a", href=True)
            link_map = {}

            for link in all_links:
                text = link.get_text().strip()
                href = link.get("href")
                if text and href:
                    link_map[text] = href

            # Extract entries using line-by-line analysis
            rank_pattern = re.compile(r"(\d+)位")
            current_rank = None

            for i, line in enumerate(lines):
                rank_match = rank_pattern.search(line)
                if rank_match:
                    rank = int(rank_match.group(1))

                    # Get title (might be in same line or next line)
                    title_part = rank_pattern.sub("", line).strip()
                    if not title_part and i + 1 < len(lines):
                        title_part = lines[i + 1].strip()

                    if title_part and title_part not in ["順位", "タイトル"]:
                        # Clean up title
                        title = title_part

                        # Try to find matching URL
                        url_link = ""
                        hatena_url = ""

                        # Look for exact match or partial match in link_map
                        for link_text, href in link_map.items():
                            if (
                                title in link_text
                                or link_text in title
                                or self.similarity_score(title, link_text) > 0.8
                            ):
                                if (
                                    "bookmark.hatena.ne.jp" in href
                                    or "b.hatena.ne.jp" in href
                                ):
                                    hatena_url = href
                                elif (
                                    href.startswith("http")
                                    and "hatenastaff.com" not in href
                                ):
                                    url_link = href
                                break

                        if rank <= expected_count:
                            entries.append(
                                {
                                    "rank": rank,
                                    "title": title,
                                    "url": url_link,
                                    "hatena_url": hatena_url,
                                }
                            )

                            if len(entries) >= expected_count:
                                break

        # Sort by rank and limit to expected count
        entries.sort(key=lambda x: x["rank"])
        entries = entries[:expected_count]

        logger.info(f"Extracted {len(entries)} entries from {url}")

        # Log first few entries for debugging
        for i, entry in enumerate(entries[:3]):
            logger.debug(
                f"Entry {i + 1}: Rank={entry['rank']}, Title='{entry['title'][:50]}...', URL='{entry['url'][:50]}...', Hatena='{entry['hatena_url'][:50]}...'"
            )

        return entries

    def similarity_score(self, text1: str, text2: str) -> float:
        """Calculate similarity score between two texts"""
        text1_words = set(text1.lower().split())
        text2_words = set(text2.lower().split())

        if not text1_words or not text2_words:
            return 0.0

        intersection = text1_words.intersection(text2_words)
        union = text1_words.union(text2_words)

        return len(intersection) / len(union) if union else 0.0

    def extract_archive_links(self) -> Dict[str, List[tuple]]:
        """Extract weekly and monthly ranking article links from archive page"""
        soup = self.get_page(self.archive_url)
        if not soup:
            return {"weekly": [], "monthly": []}

        weekly_links = []
        monthly_links = []

        # Find all links in the archive page
        for link in soup.find_all("a", href=True):
            href = link.get("href")
            text = link.get_text().strip()

            if not href or not text:
                continue

            # Make absolute URL
            if href.startswith("/"):
                href = self.base_url + href

            # Check for weekly ranking pattern
            if "今週のはてなブックマーク数ランキング（2025年" in text:
                match = re.search(r"2025年(\d+)月第(\d+)週", text)
                if match:
                    month = int(match.group(1))
                    week = int(match.group(2))
                    if 1 <= month <= 5:
                        weekly_links.append((href, month, week))

            # Check for monthly ranking pattern
            elif "月間はてなブックマーク数ランキング（2025年" in text:
                match = re.search(r"2025年(\d+)月", text)
                if match:
                    month = int(match.group(1))
                    if 1 <= month <= 5:
                        monthly_links.append((href, month))

        logger.info(
            f"Found {len(weekly_links)} weekly links and {len(monthly_links)} monthly links"
        )
        return {"weekly": weekly_links, "monthly": monthly_links}

    def rescrape_specific_files(
        self, weekly_files: List[str] = None, monthly_files: List[str] = None
    ):
        """Re-scrape specific files"""

        if weekly_files is None:
            weekly_files = []
        if monthly_files is None:
            monthly_files = []

        # Get all archive links
        links = self.extract_archive_links()

        # Re-scrape weekly files
        if weekly_files:
            logger.info(f"Re-scraping {len(weekly_files)} weekly files...")
            for url, month, week in links["weekly"]:
                filename = f"2025-{month:02d}-{week}.json"
                if filename in weekly_files:
                    filepath = os.path.join(self.week_dir, filename)
                    logger.info(
                        f"Re-scraping weekly data: {month}月第{week}週 -> {filename}"
                    )

                    entries = self.extract_ranking_data_improved(url, 30)

                    if entries and len(entries) == 30:
                        with open(filepath, "w", encoding="utf-8") as f:
                            json.dump(entries, f, ensure_ascii=False, indent=2)
                        logger.info(f"Re-saved {len(entries)} entries to {filename}")
                    else:
                        logger.warning(
                            f"Failed to get complete data for {filename}: got {len(entries)} entries"
                        )

                    time.sleep(2)

        # Re-scrape monthly files
        if monthly_files:
            logger.info(f"Re-scraping {len(monthly_files)} monthly files...")
            for url, month in links["monthly"]:
                filename = f"2025-{month:02d}.json"
                if filename in monthly_files:
                    filepath = os.path.join(self.month_dir, filename)
                    logger.info(f"Re-scraping monthly data: {month}月 -> {filename}")

                    entries = self.extract_ranking_data_improved(url, 50)

                    if entries and len(entries) == 50:
                        with open(filepath, "w", encoding="utf-8") as f:
                            json.dump(entries, f, ensure_ascii=False, indent=2)
                        logger.info(f"Re-saved {len(entries)} entries to {filename}")
                    else:
                        logger.warning(
                            f"Failed to get complete data for {filename}: got {len(entries)} entries"
                        )

                    time.sleep(2)

    def rescrape_all_with_urls(self):
        """Re-scrape all files with improved URL extraction"""
        logger.info("Re-scraping all files with improved URL extraction...")

        # Get all archive links
        links = self.extract_archive_links()

        # Re-scrape all weekly data
        logger.info("Re-scraping all weekly ranking data...")
        for url, month, week in links["weekly"]:
            filename = f"2025-{month:02d}-{week}.json"
            filepath = os.path.join(self.week_dir, filename)

            logger.info(f"Re-scraping weekly data: {month}月第{week}週 -> {filename}")
            entries = self.extract_ranking_data_improved(url, 30)

            if entries:
                with open(filepath, "w", encoding="utf-8") as f:
                    json.dump(entries, f, ensure_ascii=False, indent=2)
                logger.info(f"Saved {len(entries)} entries to {filename}")
            else:
                logger.warning(f"No entries found for {filename}")

            time.sleep(2)

        # Re-scrape all monthly data
        logger.info("Re-scraping all monthly ranking data...")
        for url, month in links["monthly"]:
            filename = f"2025-{month:02d}.json"
            filepath = os.path.join(self.month_dir, filename)

            logger.info(f"Re-scraping monthly data: {month}月 -> {filename}")
            entries = self.extract_ranking_data_improved(url, 50)

            if entries:
                with open(filepath, "w", encoding="utf-8") as f:
                    json.dump(entries, f, ensure_ascii=False, indent=2)
                logger.info(f"Saved {len(entries)} entries to {filename}")
            else:
                logger.warning(f"No entries found for {filename}")

            time.sleep(2)

        logger.info("Re-scraping completed!")


if __name__ == "__main__":
    scraper = ImprovedHatenaBookmarkScraper()

    # Re-scrape all files with improved URL extraction
    scraper.rescrape_all_with_urls()



#!/usr/bin/env python3
"""
Extract table content from the debug HTML file
"""

from bs4 import BeautifulSoup


def extract_table_content():
    """Extract and analyze table content"""

    with open(
        "/Users/go/Desktop/dev/mcp-servers/src/hatena-weekly/tool/debug_page.html",
        "r",
        encoding="utf-8",
    ) as f:
        content = f.read()

    soup = BeautifulSoup(content, "html.parser")

    tables = soup.find_all("table")
    print(f"Found {len(tables)} tables")

    for i, table in enumerate(tables):
        print(f"\n--- TABLE {i + 1} ---")

        rows = table.find_all("tr")
        print(f"Rows: {len(rows)}")

        for j, row in enumerate(rows[:10]):  # Show first 10 rows
            cells = row.find_all(["td", "th"])
            print(f"\nRow {j + 1} ({len(cells)} cells):")

            for k, cell in enumerate(cells):
                cell_text = cell.get_text().strip()

                # Check for links in the cell
                links = cell.find_all("a", href=True)
                link_info = ""
                if links:
                    link_info = f" [Links: {len(links)}]"
                    for link in links[:2]:  # Show first 2 links
                        href = link.get("href")
                        link_text = link.get_text().strip()
                        link_info += f"\n    -> {link_text[:30]}... => {href[:60]}..."

                print(f"  Cell {k + 1}: {cell_text[:100]}...{link_info}")

        # Print the raw HTML of the table for analysis
        print(f"\n--- RAW HTML OF TABLE {i + 1} (first 2000 chars) ---")
        print(str(table)[:2000])


if __name__ == "__main__":
    extract_table_content()



#!/usr/bin/env python3
"""
Test script for Hatena Bookmark MCP server core functions
"""

import sys
from pathlib import Path

# Add the src directory to the Python path
src_path = Path(__file__).parent.parent / "src"
sys.path.insert(0, str(src_path))

from mcp_server_hatena_weekly.server import load_weekly_data, load_monthly_data


def test_weekly_data_loading():
    """Test the weekly data loading function"""
    print("=== Testing Weekly Data Loading ===")

    # Test valid data
    try:
        print("Testing 2025年3月第2週...")
        result = load_weekly_data(2025, 3, 2)
        print(f"✓ Success: Got {len(result)} entries")
        print(f"First entry: {result[0].rank}位 - {result[0].title[:60]}...")
        print(f"URL: {result[0].url}")
        print(f"Hatena URL: {result[0].hatena_url}")

    except Exception as e:
        print(f"✗ Error: {e}")

    # Test another valid data
    try:
        print("\nTesting 2025年1月第1週...")
        result = load_weekly_data(2025, 1, 1)
        print(f"✓ Success: Got {len(result)} entries")
        print(f"First entry: {result[0].rank}位 - {result[0].title[:60]}...")

    except Exception as e:
        print(f"✗ Error: {e}")

    # Test invalid data (should fail)
    try:
        print("\nTesting invalid data (2025年1月第6週)...")
        result = load_weekly_data(2025, 1, 6)
        print(f"✗ Unexpected success: Got {len(result)} entries")

    except Exception as e:
        print(f"✓ Expected error: {e}")


def test_monthly_data_loading():
    """Test the monthly data loading function"""
    print("\n=== Testing Monthly Data Loading ===")

    # Test valid data
    try:
        print("Testing 2025年3月...")
        result = load_monthly_data(2025, 3)
        print(f"✓ Success: Got {len(result)} entries")
        print(f"First entry: {result[0].rank}位 - {result[0].title[:60]}...")
        print(f"URL: {result[0].url}")
        print(f"Hatena URL: {result[0].hatena_url}")

    except Exception as e:
        print(f"✗ Error: {e}")

    # Test another valid data
    try:
        print("\nTesting 2025年1月...")
        result = load_monthly_data(2025, 1)
        print(f"✓ Success: Got {len(result)} entries")
        print(f"First entry: {result[0].rank}位 - {result[0].title[:60]}...")

    except Exception as e:
        print(f"✗ Error: {e}")

    # Test invalid month (should fail)
    try:
        print("\nTesting invalid month (2025年6月)...")
        result = load_monthly_data(2025, 6)
        print(f"✗ Unexpected success: Got {len(result)} entries")

    except Exception as e:
        print(f"✓ Expected error: {e}")


def test_data_quality():
    """Test data quality across all files"""
    print("\n=== Testing Data Quality ===")

    total_weekly_tested = 0
    total_monthly_tested = 0

    # Test all weekly data
    print("Testing all weekly data...")
    for month in range(1, 6):  # 1-5
        max_week = 5 if month == 3 else 4  # March has 5 weeks
        for week in range(1, max_week + 1):
            try:
                result = load_weekly_data(2025, month, week)
                if len(result) == 30:
                    total_weekly_tested += 1
                    # Check data completeness
                    urls_count = sum(1 for entry in result if entry.url)
                    hatena_urls_count = sum(1 for entry in result if entry.hatena_url)
                    print(
                        f"✓ 2025-{month:02d}-{week}: {len(result)} entries, {urls_count} URLs, {hatena_urls_count} hatena URLs"
                    )
                else:
                    print(
                        f"✗ 2025-{month:02d}-{week}: Expected 30, got {len(result)} entries"
                    )

            except Exception as e:
                print(f"✗ 2025-{month:02d}-{week}: Error - {e}")

    # Test all monthly data
    print("\nTesting all monthly data...")
    for month in range(1, 6):  # 1-5
        try:
            result = load_monthly_data(2025, month)
            if len(result) == 50:
                total_monthly_tested += 1
                # Check data completeness
                urls_count = sum(1 for entry in result if entry.url)
                hatena_urls_count = sum(1 for entry in result if entry.hatena_url)
                print(
                    f"✓ 2025-{month:02d}: {len(result)} entries, {urls_count} URLs, {hatena_urls_count} hatena URLs"
                )
            else:
                print(f"✗ 2025-{month:02d}: Expected 50, got {len(result)} entries")

        except Exception as e:
            print(f"✗ 2025-{month:02d}: Error - {e}")

    print("\n=== Summary ===")
    print(f"Weekly files tested successfully: {total_weekly_tested}")
    print(f"Monthly files tested successfully: {total_monthly_tested}")

    return total_weekly_tested, total_monthly_tested


def test_server_startup():
    """Test if the server can be imported and initialized"""
    print("\n=== Testing Server Startup ===")

    try:
        from mcp_server_hatena_weekly.server import mcp

        print("✓ Server module imported successfully")

        # Check if tools are registered
        tools = mcp.list_tools()
        print(f"✓ Found {len(tools)} tools registered:")
        for tool in tools:
            print(f"  - {tool['name']}: {tool['description'][:60]}...")

        return True

    except Exception as e:
        print(f"✗ Server startup error: {e}")
        return False


def main():
    """Main test function"""
    print("🧪 Testing Hatena Bookmark MCP Server Core Functions")
    print("=" * 60)

    test_weekly_data_loading()
    test_monthly_data_loading()
    weekly_success, monthly_success = test_data_quality()
    server_success = test_server_startup()

    print("\n🎯 Final Results:")
    print(f"Weekly data files working: {weekly_success}/21")
    print(f"Monthly data files working: {monthly_success}/5")
    print(f"Server startup: {'✓' if server_success else '✗'}")

    if weekly_success == 21 and monthly_success == 5 and server_success:
        print("🎉 All tests passed! MCP Server is ready to use.")
    else:
        print("⚠️  Some tests failed. Please check the issues above.")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
Check data file counts and identify files that need re-scraping
"""

import os
import json
import glob


def check_data_counts():
    """Check the count of entries in each data file"""

    base_dir = os.path.join(os.path.dirname(__file__), "..", "data")
    week_dir = os.path.join(base_dir, "week")
    month_dir = os.path.join(base_dir, "month")

    print("=== Weekly Data Check (Expected: 30 entries each) ===")
    week_files = glob.glob(os.path.join(week_dir, "*.json"))
    week_files.sort()

    insufficient_weekly = []
    for file_path in week_files:
        filename = os.path.basename(file_path)
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                data = json.load(f)
                count = len(data)
                status = "✓" if count == 30 else "✗"
                print(f"{status} {filename}: {count} entries")
                if count < 30:
                    insufficient_weekly.append(filename)
        except Exception as e:
            print(f"✗ {filename}: Error reading file - {e}")
            insufficient_weekly.append(filename)

    print("\n=== Monthly Data Check (Expected: 50 entries each) ===")
    month_files = glob.glob(os.path.join(month_dir, "*.json"))
    month_files.sort()

    insufficient_monthly = []
    for file_path in month_files:
        filename = os.path.basename(file_path)
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                data = json.load(f)
                count = len(data)
                status = "✓" if count == 50 else "✗"
                print(f"{status} {filename}: {count} entries")
                if count < 50:
                    insufficient_monthly.append(filename)
        except Exception as e:
            print(f"✗ {filename}: Error reading file - {e}")
            insufficient_monthly.append(filename)

    print("\n=== Summary ===")
    print(f"Weekly files needing re-scraping: {len(insufficient_weekly)}")
    if insufficient_weekly:
        print(f"Files: {', '.join(insufficient_weekly)}")

    print(f"Monthly files needing re-scraping: {len(insufficient_monthly)}")
    if insufficient_monthly:
        print(f"Files: {', '.join(insufficient_monthly)}")

    return insufficient_weekly, insufficient_monthly


if __name__ == "__main__":
    check_data_counts()



#!/usr/bin/env python3
"""
Hatena Bookmark Weekly/Monthly Ranking Data Scraper

This script scrapes Hatena Bookmark ranking data from bookmark.hatenastaff.com
and saves it as JSON files for use with the MCP server.
"""

import os
import json
import re
import time
import requests
from bs4 import BeautifulSoup
from typing import List, Dict, Optional
import logging

# Setup logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)


class HatenaBookmarkScraper:
    def __init__(self):
        self.base_url = "https://bookmark.hatenastaff.com"
        self.archive_url = f"{self.base_url}/archive/category/ランキング"
        self.headers = {
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36"
        }
        self.session = requests.Session()
        self.session.headers.update(self.headers)

        # Create data directories
        self.data_dir = os.path.join(os.path.dirname(__file__), "..", "data")
        self.week_dir = os.path.join(self.data_dir, "week")
        self.month_dir = os.path.join(self.data_dir, "month")

        os.makedirs(self.week_dir, exist_ok=True)
        os.makedirs(self.month_dir, exist_ok=True)

    def get_page(self, url: str) -> Optional[BeautifulSoup]:
        """Get a page and return BeautifulSoup object"""
        try:
            logger.info(f"Fetching: {url}")
            response = self.session.get(url, timeout=30)
            response.raise_for_status()
            return BeautifulSoup(response.content, "html.parser")
        except Exception as e:
            logger.error(f"Error fetching {url}: {e}")
            return None

    def extract_archive_links(self) -> Dict[str, List[str]]:
        """Extract weekly and monthly ranking article links from archive page"""
        soup = self.get_page(self.archive_url)
        if not soup:
            return {"weekly": [], "monthly": []}

        weekly_links = []
        monthly_links = []

        # Find all links in the archive page
        for link in soup.find_all("a", href=True):
            href = link.get("href")
            text = link.get_text().strip()

            if not href or not text:
                continue

            # Make absolute URL
            if href.startswith("/"):
                href = self.base_url + href

            # Check for weekly ranking pattern
            if "今週のはてなブックマーク数ランキング（2025年" in text:
                # Extract month and week from text like "今週のはてなブックマーク数ランキング（2025年3月第2週）"
                match = re.search(r"2025年(\d+)月第(\d+)週", text)
                if match:
                    month = int(match.group(1))
                    week = int(match.group(2))
                    if 1 <= month <= 5:  # Only collect January to May
                        weekly_links.append((href, month, week))

            # Check for monthly ranking pattern
            elif "月間はてなブックマーク数ランキング（2025年" in text:
                # Extract month from text like "月間はてなブックマーク数ランキング（2025年2月）"
                match = re.search(r"2025年(\d+)月", text)
                if match:
                    month = int(match.group(1))
                    if 1 <= month <= 5:  # Only collect January to May
                        monthly_links.append((href, month))

        logger.info(
            f"Found {len(weekly_links)} weekly links and {len(monthly_links)} monthly links"
        )
        return {"weekly": weekly_links, "monthly": monthly_links}

    def extract_ranking_data(
        self, url: str, expected_count: int
    ) -> List[Dict[str, str]]:
        """Extract ranking data from a ranking page"""
        soup = self.get_page(url)
        if not soup:
            return []

        entries = []

        # Look for ranking entries in the page
        # The pattern seems to be: rank, title, and links are in specific structures
        # We need to find the actual ranking table/list

        # Try different selectors to find the ranking data
        # Looking for patterns like "1位", "2位", etc.
        rank_pattern = re.compile(r"(\d+)位")

        # Find all text that contains rank information
        text_content = soup.get_text()
        lines = [line.strip() for line in text_content.split("\n") if line.strip()]

        current_rank = None
        current_title = None

        for i, line in enumerate(lines):
            # Check if this line contains a rank
            rank_match = rank_pattern.search(line)
            if rank_match:
                rank = int(rank_match.group(1))

                # Look for the title in the same line or next lines
                title_part = rank_pattern.sub("", line).strip()
                if title_part:
                    current_title = title_part
                else:
                    # Title might be in the next line
                    if i + 1 < len(lines):
                        current_title = lines[i + 1].strip()

                if current_title and current_title not in ["順位", "タイトル"]:
                    entries.append(
                        {
                            "rank": rank,
                            "title": current_title,
                            "url": "",  # We'll try to find this
                            "hatena_url": "",  # We'll try to find this
                        }
                    )

                    if len(entries) >= expected_count:
                        break

        # Now try to find URLs for the entries
        # Look for all links in the page
        links = soup.find_all("a", href=True)

        for entry in entries:
            title = entry["title"]
            # Try to find a link that matches this title
            for link in links:
                link_text = link.get_text().strip()
                if title in link_text or link_text in title:
                    href = link.get("href")
                    if href:
                        if href.startswith("http") and "bookmark.hatena" not in href:
                            entry["url"] = href
                        elif "bookmark.hatena" in href or href.startswith("/"):
                            if href.startswith("/"):
                                entry["hatena_url"] = (
                                    "https://bookmark.hatena.ne.jp" + href
                                )
                            else:
                                entry["hatena_url"] = href
                    break

        logger.info(f"Extracted {len(entries)} entries from {url}")
        return entries[:expected_count]  # Ensure we don't exceed expected count

    def scrape_weekly_data(self, links: List[tuple]) -> None:
        """Scrape weekly ranking data"""
        for url, month, week in links:
            filename = f"2025-{month:02d}-{week}.json"
            filepath = os.path.join(self.week_dir, filename)

            if os.path.exists(filepath):
                logger.info(f"Skipping {filename} - already exists")
                continue

            logger.info(f"Scraping weekly data: {month}月第{week}週")
            entries = self.extract_ranking_data(url, 30)  # Weekly has 30 entries

            if entries:
                with open(filepath, "w", encoding="utf-8") as f:
                    json.dump(entries, f, ensure_ascii=False, indent=2)
                logger.info(f"Saved {len(entries)} entries to {filename}")
            else:
                logger.warning(f"No entries found for {filename}")

            time.sleep(2)  # Be respectful to the server

    def scrape_monthly_data(self, links: List[tuple]) -> None:
        """Scrape monthly ranking data"""
        for url, month in links:
            filename = f"2025-{month:02d}.json"
            filepath = os.path.join(self.month_dir, filename)

            if os.path.exists(filepath):
                logger.info(f"Skipping {filename} - already exists")
                continue

            logger.info(f"Scraping monthly data: {month}月")
            entries = self.extract_ranking_data(url, 50)  # Monthly has 50 entries

            if entries:
                with open(filepath, "w", encoding="utf-8") as f:
                    json.dump(entries, f, ensure_ascii=False, indent=2)
                logger.info(f"Saved {len(entries)} entries to {filename}")
            else:
                logger.warning(f"No entries found for {filename}")

            time.sleep(2)  # Be respectful to the server

    def run(self):
        """Main scraping function"""
        logger.info("Starting Hatena Bookmark ranking data scraping...")

        # Get all archive links
        links = self.extract_archive_links()

        # Scrape weekly data
        if links["weekly"]:
            logger.info("Scraping weekly ranking data...")
            self.scrape_weekly_data(links["weekly"])

        # Scrape monthly data
        if links["monthly"]:
            logger.info("Scraping monthly ranking data...")
            self.scrape_monthly_data(links["monthly"])

        logger.info("Scraping completed!")


if __name__ == "__main__":
    scraper = HatenaBookmarkScraper()
    scraper.run()



#!/usr/bin/env python3
"""
Debug script to analyze the actual HTML structure of Hatena ranking pages
"""

import requests
from bs4 import BeautifulSoup
import re


def analyze_page_structure(url):
    """Analyze the structure of a ranking page"""
    headers = {
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36"
    }

    print(f"Analyzing: {url}")
    response = requests.get(url, headers=headers)
    soup = BeautifulSoup(response.content, "html.parser")

    print("=" * 80)
    print("PAGE TITLE:")
    print(soup.title.text if soup.title else "No title")

    print("\n" + "=" * 80)
    print("LOOKING FOR RANKING PATTERNS:")

    # Look for rank patterns
    text_content = soup.get_text()
    rank_patterns = re.findall(r"\d+位.*", text_content)

    print(f"Found {len(rank_patterns)} rank patterns:")
    for i, pattern in enumerate(rank_patterns[:10]):  # Show first 10
        print(f"{i + 1}. {pattern.strip()}")

    print("\n" + "=" * 80)
    print("LOOKING FOR TABLES:")
    tables = soup.find_all("table")
    print(f"Found {len(tables)} tables")

    for i, table in enumerate(tables):
        print(f"\nTable {i + 1}:")
        rows = table.find_all("tr")
        print(f"  Rows: {len(rows)}")
        if rows:
            first_row = rows[0]
            cells = first_row.find_all(["td", "th"])
            print(f"  First row cells: {len(cells)}")
            for j, cell in enumerate(cells[:3]):
                print(f"    Cell {j + 1}: {cell.get_text().strip()[:50]}...")

    print("\n" + "=" * 80)
    print("LOOKING FOR LISTS:")
    lists = soup.find_all(["ol", "ul"])
    print(f"Found {len(lists)} lists")

    for i, lst in enumerate(lists):
        items = lst.find_all("li")
        print(f"List {i + 1}: {len(items)} items")
        if items:
            print(f"  First item: {items[0].get_text().strip()[:100]}...")

    print("\n" + "=" * 80)
    print("LOOKING FOR LINKS:")
    links = soup.find_all("a", href=True)
    external_links = [
        link
        for link in links
        if link.get("href").startswith("http")
        and "hatenastaff.com" not in link.get("href")
    ]

    print(f"Total links: {len(links)}")
    print(f"External links: {len(external_links)}")

    print("\nFirst 10 external links:")
    for i, link in enumerate(external_links[:10]):
        href = link.get("href")
        text = link.get_text().strip()
        print(f"{i + 1}. {text[:50]}... -> {href[:60]}...")

    print("\n" + "=" * 80)
    print("LOOKING FOR DIVS WITH RANKING DATA:")

    # Look for specific div structures
    entry_content = soup.find("div", class_="entry-content")
    if entry_content:
        print("Found entry-content div")
        # Look for patterns within entry content
        text = entry_content.get_text()
        lines = [line.strip() for line in text.split("\n") if line.strip()]

        rank_lines = [line for line in lines if re.search(r"\d+位", line)]
        print(f"Rank lines in entry-content: {len(rank_lines)}")

        for i, line in enumerate(rank_lines[:10]):
            print(f"  {i + 1}. {line}")

    return soup


if __name__ == "__main__":
    # Test with a specific ranking page
    test_url = "https://bookmark.hatenastaff.com/entry/2025/03/11/113220"
    soup = analyze_page_structure(test_url)

    # Save the full HTML for manual inspection
    with open(
        "/Users/go/Desktop/dev/mcp-servers/src/hatena-weekly/tool/debug_page.html",
        "w",
        encoding="utf-8",
    ) as f:
        f.write(str(soup))
    print("\nFull HTML saved to debug_page.html")



import logging

from mcp_server_hatena_weekly.server import mcp


def main():
    """
    Hatena Bookmark Weekly/Monthly Ranking Server - Model Context Protocol server
    for fetching Hatena bookmark ranking data

    This server provides access to Hatena bookmark ranking data for 2025 (January-May).

    Available features:
    1. weekly: Get weekly Hatena bookmark ranking data (30 entries)
    2. monthly: Get monthly Hatena bookmark ranking data (50 entries)
    """
    import argparse

    # Command line argument configuration
    parser = argparse.ArgumentParser(
        description="Start the Hatena Bookmark Weekly/Monthly Ranking server. "
        "Fetch ranking data from pre-scraped Hatena bookmark rankings."
    )
    parser.add_argument(
        "--sse",
        choices=["on", "off"],
        default="off",
        help='Enable SSE transport when set to "on"',
    )
    parser.add_argument(
        "--host",
        default="localhost",
        help="Host to bind the server to (default: localhost)",
    )
    parser.add_argument(
        "--port",
        type=int,
        default=8000,
        help="Port to bind the server to (default: 8000)",
    )
    parser.add_argument(
        "--log-level",
        choices=["debug", "info", "warning", "error"],
        default="info",
        help="Set logging level:\n"
        "  debug: Detailed debug information\n"
        "  info: General execution information (default)\n"
        "  warning: Potential issues that do not affect execution\n"
        "  error: Errors that occur during execution",
    )
    args = parser.parse_args()

    # Map string log levels to logging constants
    log_level_map = {
        "debug": logging.DEBUG,
        "info": logging.INFO,
        "warning": logging.WARNING,
        "error": logging.ERROR,
    }

    log_level = log_level_map.get(args.log_level, logging.INFO)
    if args.log_level not in log_level_map:
        raise ValueError(
            f"Invalid log level: {args.log_level}. Choose from {list(log_level_map.keys())}."
        )

    # Configure logging based on command line argument
    logging.basicConfig(
        level=log_level, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )

    # Run server with configured arguments
    if args.sse == "on":
        mcp.run(transport="sse", host=args.host, port=args.port)
    else:
        mcp.run()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
Test script for Hatena Bookmark MCP server
"""

import asyncio
import sys

from pathlib import Path

# Add the src directory to the Python path
src_path = Path(__file__).parent.parent / "src"
sys.path.insert(0, str(src_path))

from mcp_server_hatena_weekly.server import weekly, monthly


async def test_weekly():
    """Test the weekly tool"""
    print("=== Testing Weekly Tool ===")

    # Test valid data
    try:
        print("Testing 2025年3月第2週...")
        result = await weekly(year=2025, month=3, week=2)
        print(f"✓ Success: Got {len(result)} entries")
        print(f"First entry: {result[0].rank}位 - {result[0].title[:60]}...")
        print(f"URL: {result[0].url}")
        print(f"Hatena URL: {result[0].hatena_url}")

    except Exception as e:
        print(f"✗ Error: {e}")

    # Test another valid data
    try:
        print("\nTesting 2025年1月第1週...")
        result = await weekly(year=2025, month=1, week=1)
        print(f"✓ Success: Got {len(result)} entries")
        print(f"First entry: {result[0].rank}位 - {result[0].title[:60]}...")

    except Exception as e:
        print(f"✗ Error: {e}")

    # Test invalid data (should fail)
    try:
        print("\nTesting invalid data (2025年1月第6週)...")
        result = await weekly(year=2025, month=1, week=6)
        print(f"✗ Unexpected success: Got {len(result)} entries")

    except Exception as e:
        print(f"✓ Expected error: {e}")

    # Test invalid year (should fail)
    try:
        print("\nTesting invalid year (2024年1月第1週)...")
        result = await weekly(year=2024, month=1, week=1)
        print(f"✗ Unexpected success: Got {len(result)} entries")

    except Exception as e:
        print(f"✓ Expected error: {e}")


async def test_monthly():
    """Test the monthly tool"""
    print("\n=== Testing Monthly Tool ===")

    # Test valid data
    try:
        print("Testing 2025年3月...")
        result = await monthly(year=2025, month=3)
        print(f"✓ Success: Got {len(result)} entries")
        print(f"First entry: {result[0].rank}位 - {result[0].title[:60]}...")
        print(f"URL: {result[0].url}")
        print(f"Hatena URL: {result[0].hatena_url}")

    except Exception as e:
        print(f"✗ Error: {e}")

    # Test another valid data
    try:
        print("\nTesting 2025年1月...")
        result = await monthly(year=2025, month=1)
        print(f"✓ Success: Got {len(result)} entries")
        print(f"First entry: {result[0].rank}位 - {result[0].title[:60]}...")

    except Exception as e:
        print(f"✗ Error: {e}")

    # Test invalid month (should fail)
    try:
        print("\nTesting invalid month (2025年6月)...")
        result = await monthly(year=2025, month=6)
        print(f"✗ Unexpected success: Got {len(result)} entries")

    except Exception as e:
        print(f"✓ Expected error: {e}")


async def test_data_quality():
    """Test data quality across all files"""
    print("\n=== Testing Data Quality ===")

    total_weekly_tested = 0
    total_monthly_tested = 0

    # Test all weekly data
    print("Testing all weekly data...")
    for month in range(1, 6):  # 1-5
        max_week = 5 if month == 3 else 4  # March has 5 weeks
        for week in range(1, max_week + 1):
            try:
                result = await weekly(year=2025, month=month, week=week)
                if len(result) == 30:
                    total_weekly_tested += 1
                    print(f"✓ 2025-{month:02d}-{week}: {len(result)} entries")
                else:
                    print(
                        f"✗ 2025-{month:02d}-{week}: Expected 30, got {len(result)} entries"
                    )

            except Exception as e:
                print(f"✗ 2025-{month:02d}-{week}: Error - {e}")

    # Test all monthly data
    print("\nTesting all monthly data...")
    for month in range(1, 6):  # 1-5
        try:
            result = await monthly(year=2025, month=month)
            if len(result) == 50:
                total_monthly_tested += 1
                print(f"✓ 2025-{month:02d}: {len(result)} entries")
            else:
                print(f"✗ 2025-{month:02d}: Expected 50, got {len(result)} entries")

        except Exception as e:
            print(f"✗ 2025-{month:02d}: Error - {e}")

    print("\n=== Summary ===")
    print(f"Weekly files tested successfully: {total_weekly_tested}")
    print(f"Monthly files tested successfully: {total_monthly_tested}")


async def main():
    """Main test function"""
    print("🧪 Testing Hatena Bookmark MCP Server")
    print("=" * 50)

    await test_weekly()
    await test_monthly()
    await test_data_quality()

    print("\n🎉 Testing completed!")


if __name__ == "__main__":
    asyncio.run(main())



import json
import logging
from pathlib import Path
from typing import List

from fastmcp import FastMCP
from mcp.shared.exceptions import McpError
from mcp.types import INTERNAL_ERROR, INVALID_PARAMS, ErrorData
from pydantic import BaseModel, ConfigDict, Field

# Configure logging
logger = logging.getLogger(__name__)

# Create an MCP server instance
mcp = FastMCP("Hatena Bookmark Weekly/Monthly Ranking Server")


# Pydantic models
class BookmarkEntry(BaseModel):
    """Model for a bookmark entry."""

    model_config = ConfigDict(extra="forbid")

    rank: int = Field(description="Ranking position")
    title: str = Field(description="Article title")
    url: str = Field(description="Original article URL")
    hatena_url: str = Field(description="Hatena bookmark URL")


class WeeklyRequest(BaseModel):
    """Model for weekly ranking request parameters."""

    model_config = ConfigDict(extra="forbid")

    year: int = Field(description="Year (2025 only)", ge=2025, le=2025)
    month: int = Field(description="Month (1-5)", ge=1, le=5)
    week: int = Field(description="Week number (1-5)", ge=1, le=5)


class MonthlyRequest(BaseModel):
    """Model for monthly ranking request parameters."""

    model_config = ConfigDict(extra="forbid")

    year: int = Field(description="Year (2025 only)", ge=2025, le=2025)
    month: int = Field(description="Month (1-5)", ge=1, le=5)


def get_data_directory() -> Path:
    """Get the data directory path"""
    # Get the directory where this file is located
    current_file = Path(__file__)
    # Go up to src/mcp_server_hatena_weekly, then to project root, then to data
    data_dir = current_file.parent.parent.parent / "data"
    return data_dir


def load_weekly_data(year: int, month: int, week: int) -> List[BookmarkEntry]:
    """Load weekly ranking data from JSON file"""
    data_dir = get_data_directory()
    file_path = data_dir / "week" / f"{year}-{month:02d}-{week}.json"

    if not file_path.exists():
        raise McpError(
            ErrorData(
                code=INVALID_PARAMS,
                message=f"Weekly data not found for {year}年{month}月第{week}週. File: {file_path}",
            )
        )

    try:
        with open(file_path, "r", encoding="utf-8") as f:
            data = json.load(f)

        entries = []
        for item in data:
            entries.append(
                BookmarkEntry(
                    rank=item["rank"],
                    title=item["title"],
                    url=item["url"],
                    hatena_url=item["hatena_url"],
                )
            )

        logger.info(f"Loaded {len(entries)} weekly entries for {year}-{month:02d}-{week}")
        return entries

    except Exception as e:
        logger.error(f"Error loading weekly data from {file_path}: {e}")
        raise McpError(
            ErrorData(code=INTERNAL_ERROR, message=f"Failed to load weekly data: {str(e)}")
        )


def load_monthly_data(year: int, month: int) -> List[BookmarkEntry]:
    """Load monthly ranking data from JSON file"""
    data_dir = get_data_directory()
    file_path = data_dir / "month" / f"{year}-{month:02d}.json"

    if not file_path.exists():
        raise McpError(
            ErrorData(
                code=INVALID_PARAMS,
                message=f"Monthly data not found for {year}年{month}月. File: {file_path}",
            )
        )

    try:
        with open(file_path, "r", encoding="utf-8") as f:
            data = json.load(f)

        entries = []
        for item in data:
            entries.append(
                BookmarkEntry(
                    rank=item["rank"],
                    title=item["title"],
                    url=item["url"],
                    hatena_url=item["hatena_url"],
                )
            )

        logger.info(f"Loaded {len(entries)} monthly entries for {year}-{month:02d}")
        return entries

    except Exception as e:
        logger.error(f"Error loading monthly data from {file_path}: {e}")
        raise McpError(
            ErrorData(code=INTERNAL_ERROR, message=f"Failed to load monthly data: {str(e)}")
        )


@mcp.tool()
async def weekly(year: int, month: int, week: int) -> List[BookmarkEntry]:
    """
    週次人気はてなブックマーク情報の取得

    Get weekly popular Hatena bookmark ranking data.

    Args:
        year: 年度 (2025年のみ対応)
        month: 月 (1-5)
        week: 週番号 (1-5、5週目が存在しない月もあります)

    Returns:
        List of BookmarkEntry models containing ranking data with:
        - rank: ランキング順位
        - title: 記事タイトル
        - url: 元記事のURL
        - hatena_url: はてなブックマークURL

        週次ランキングは30件のエントリーを返します。

    Raises:
        McpError: パラメータが無効、またはデータの読み込みに失敗した場合
    """
    # Validate and create request model
    try:
        request = WeeklyRequest(year=year, month=month, week=week)
    except Exception as e:
        logger.error(f"Weekly request validation failed: {str(e)}")
        raise McpError(ErrorData(code=INVALID_PARAMS, message=f"Invalid parameters: {str(e)}"))

    try:
        entries = load_weekly_data(request.year, request.month, request.week)
        return entries

    except McpError:
        raise
    except Exception as e:
        logger.error(f"Unexpected error in weekly tool: {str(e)}")
        raise McpError(
            ErrorData(
                code=INTERNAL_ERROR,
                message=f"Unexpected error while fetching weekly data: {str(e)}",
            )
        )


@mcp.tool()
async def monthly(year: int, month: int) -> List[BookmarkEntry]:
    """
    月次人気はてなブックマーク情報の取得

    Get monthly popular Hatena bookmark ranking data.

    Args:
        year: 年度 (2025年のみ対応)
        month: 月 (1-5)

    Returns:
        List of BookmarkEntry models containing ranking data with:
        - rank: ランキング順位
        - title: 記事タイトル
        - url: 元記事のURL
        - hatena_url: はてなブックマークURL

        月次ランキングは50件のエントリーを返します。

    Raises:
        McpError: パラメータが無効、またはデータの読み込みに失敗した場合
    """
    # Validate and create request model
    try:
        request = MonthlyRequest(year=year, month=month)
    except Exception as e:
        logger.error(f"Monthly request validation failed: {str(e)}")
        raise McpError(ErrorData(code=INVALID_PARAMS, message=f"Invalid parameters: {str(e)}"))

    try:
        entries = load_monthly_data(request.year, request.month)
        return entries

    except McpError:
        raise
    except Exception as e:
        logger.error(f"Unexpected error in monthly tool: {str(e)}")
        raise McpError(
            ErrorData(
                code=INTERNAL_ERROR,
                message=f"Unexpected error while fetching monthly data: {str(e)}",
            )
        )
